using System;
using System.IO;
using System.Net;
using System.Threading.Tasks;
using System.Windows.Forms;
using Inetlab.MMS;
using Inetlab.MMS.Logging;
using Inetlab.MMS.MM7;
using MM7ClientDemo.Controls;

namespace MM7ClientDemo
{
    public partial class MainForm : Form
    {

        private readonly MM7ClientCreator _clientCreator;
        private readonly VASPActions _vaspActions;
        private readonly MMSCActions _mmscActions;

        public MainForm()
        {

            InitializeComponent();

               
            string sampleFilesPath = @"..\..\Samples\files\"; 

            if (!Directory.Exists(sampleFilesPath))
            {
                sampleFilesPath = @"..\..\..\files\";
            }

            LogManager.SetLoggerFactory(new TextBoxLogFactory(tbClientLog, LogLevel.Verbose));

            lLicenseInfo.Text = Inetlab.MMS.LicenseManager.Status;

            vaspActionsControl.SendMMSFile += OnSendMMSFile;
            vaspActionsControl.SendSmilMMS += OnSendSmilMMS;
            vaspActionsControl.SendSimpleMMS += OnSendSimpleMMS;

            mmscActionsControl.SendDeliveryReport += OnSendDeliveryReport;
            mmscActionsControl.SendReadReplyReport += OnSendReadReplyReport;

            mmscActionsControl.Visible = false;
            vaspActionsControl.Visible = true;


            vaspActionsControl.SetFilesPath(sampleFilesPath);


            _clientCreator = new MM7ClientCreator(clientSettings);
            _vaspActions = new VASPActions(_clientCreator);
            _mmscActions = new MMSCActions(_clientCreator);

        }

     

        private async void OnSendSimpleMMS(SimpleMMSArgs args)
        {
            await HandleException(() => _vaspActions.SendSimpleMMS(args.Recipient, args.Subject, args.MessageText, args.ImageFilePath));
        }

        private async void OnSendSmilMMS(SmilMMSArgs args)
        {
            await HandleException(() => _vaspActions.SendMMSFromSMIL(args.Recipient, args.Subject, args.SmilFilePath));
        }

        private async void OnSendMMSFile(MMSFileArgs args)
        {
            await HandleException(() => _vaspActions.SendMMSFile(args.FilePath, args.Recipient));
        }

        private async void OnSendReadReplyReport(ReadReplyReportArgs arg)
        {
            await HandleException(() => _mmscActions.SendReadReplyReport(arg.SenderAddress, arg.Recipient));
        }

        private async void OnSendDeliveryReport(DeliveryReportArgs arg)
        {
            await HandleException(() => _mmscActions.SendDeliveryReport(arg.SenderAddress, arg.Recipient));
        }

        

        private void ModeChanged(object sender, EventArgs e)
        {
            mmscActionsControl.Visible = rbMMSC.Checked;
            vaspActionsControl.Visible = rbVASP.Checked;
        }


        private async Task HandleException(Func<Task<GenericResponse>> func)
        {

            try
            {
                GenericResponse resp = await func();


                MessageBox.Show(
                    $"MMS message has been sent. Status: {resp.Status.StatusCode} {resp.Status.StatusText}",
                    "Server response",
                    MessageBoxButtons.OK, MessageBoxIcon.Information);
            }
            catch (MM7Exception ex)
            {
                MessageBox.Show(
                    ex.Message + Environment.NewLine +
                    ex.Status.Details,
                    "Server response error",
                    MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (WebException ex)
            {
                MessageBox.Show(
                    ex.Message,
                    "Server connection error",
                    MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            catch (Exception ex)
            {
                MessageBox.Show(
                    ex.ToString(),
                    "Application error",
                    MessageBoxButtons.OK, MessageBoxIcon.Error);
            }

        }

    }
}