﻿using System;
using System.Drawing;
using System.Drawing.Imaging;
using System.IO;
using System.Text;
using System.Threading;
using System.Threading.Tasks;
using Inetlab.MMS;
using Inetlab.MMS.MM7;

namespace MM7ClientDemo
{
    public class VASPActions
    {
        private readonly MM7ClientCreator _clientCreator;

        public VASPActions(MM7ClientCreator clientCreator)
        {
            _clientCreator = clientCreator;
        }


        public async Task<GenericResponse> SendMMSFile(string filePath, string recipient, CancellationToken token = default)
        {
            if (!File.Exists(filePath))
            {
                throw new ArgumentException($"File {filePath} not found. Please specify correct MMS file path.");
            }

            // Read MMS Message from file
            MMSMessage mes;
            using (FileStream fs = new FileStream(filePath, FileMode.Open, FileAccess.Read))
            {
                mes = MMSMessage.Decode(fs);
            }

            // Change recipient
            mes.Header.To = recipient + "/TYPE=PLMN";


            MM7Client client = _clientCreator.Create();

            // Submit MMS Message as VASP
            return await client.Submit(mes, token).ConfigureAwait(false);



        }


        public async Task<GenericResponse> SendSimpleMMS(string recipient, string subject, string messageText, string imagePath, CancellationToken token = default)
        {


            if (string.IsNullOrEmpty(imagePath) && string.IsNullOrEmpty(messageText))
            {
                throw new ArgumentException("Please specify MMS Image file path or/and message text.");
            }

            // Create MMS message
            MMSMessage mes = new MMSMessage();

            mes.Header.To = recipient + "/TYPE=PLMN";
            mes.Header.ContentType = MMSContentType.MultipartMixed;
            mes.Header.ContentType.Start = "<presentation-part>";
            mes.Header.ContentType.Type2 = "application/smil";

            mes.Header.TransactionId = DateTime.Now.Ticks.ToString();
            mes.Header.Subject = subject;

            // Make MMS part from Image
            MMSPart partImage = null;

            if (!string.IsNullOrEmpty(imagePath))
            {
                if (!File.Exists(imagePath))
                {
                    throw new ArgumentException(
                        $"File {imagePath} not found. Please specify correct MMS Image file path.");
                }

                Image img = Image.FromFile(imagePath);

                partImage = new MMSPart();
                partImage.Parameters["Content-Location"] = Path.GetFileName(imagePath);
                partImage.Parameters["Content-ID"] = "<" + partImage.Parameters["Content-Location"] + ">";
                string imageContentType = GetContentTypeByImageFormat(img.RawFormat);
                if (imageContentType == null)
                {
                    throw new ArgumentException($"Unsupported image format {img.RawFormat}.");
                }

                partImage.Parameters["Content-Transfer-Encoding"] = "binary"; // or "base64";
                partImage.ContentType = new MMSContentType(imageContentType);
                partImage.ContentType.Name = partImage.Parameters["Content-Location"];

                MemoryStream imageStream = new MemoryStream();
                img.Save(imageStream, img.RawFormat);
                imageStream.Position = 0;
                partImage.Data = imageStream.ToArray();
            }

            // Make text MMS part
            MMSPart partText = null;
            if (!string.IsNullOrEmpty(messageText))
            {
                partText = new MMSPart();
                partText.Parameters["Content-ID"] = "<default.txt>";
                partText.Parameters["Content-Location"] = "default.txt";
                partText.ContentType = MMSContentType.Text;
                partText.ContentType.Charset = "utf-8";
                partText.ContentType.Name = partText.Parameters["Content-Location"];
                partText.Data = Encoding.UTF8.GetBytes(messageText);

            }

            // Make SMIL part
            MMSPart partSmil = new MMSPart();
            partSmil.Parameters["Content-ID"] = "<presentation-part>";
            partSmil.Parameters["Content-Location"] = "default.smil";
            partSmil.ContentType = MMSContentType.Smil;
            partSmil.ContentType.Name = partSmil.Parameters["Content-Location"];
            StringBuilder smilBuilder = new StringBuilder();
            smilBuilder.Append("<?xml version=\"1.0\"?>\r\n");
            smilBuilder.Append(
                "<!DOCTYPE smil PUBLIC \"-//W3C//DTD SMIL 2.0//EN\" \"http://www.w3.org/2001/SMIL20/SMIL20.dtd\">\r\n");
            smilBuilder.Append("<smil xmlns=\"http://www.w3.org/2001/SMIL20/Language\">\r\n");
            smilBuilder.Append("<head>\r\n");
            smilBuilder.Append("<layout>\r\n");
            smilBuilder.Append("<root-layout width=\"100%\" height=\"100%\" />\r\n");
            if (partImage != null && partText != null)
            {
                smilBuilder.Append("<region id=\"Image\" width=\"100%\" height=\"50%\" fit=\"meet\"/>\r\n");
            }
            else if (partImage != null)
            {
                smilBuilder.Append("<region id=\"Image\" width=\"100%\" height=\"100%\" fit=\"meet\"/>\r\n");
            }

            if (partText != null && partImage != null)
            {
                smilBuilder.Append("<region id=\"Text\" width=\"100%\" height=\"50%\" top=\"50%\" />\r\n");
            }
            else if (partText != null)
            {
                smilBuilder.Append("<region id=\"Text\" width=\"100%\" height=\"100%\" />\r\n");
            }

            smilBuilder.Append("</layout>\r\n");
            smilBuilder.Append("</head>\r\n");
            smilBuilder.Append("<body>\r\n");
            if (partImage != null)
            {
                smilBuilder.Append("<par><img src=\"" + partImage.Parameters["Content-Location"] +
                                   "\" region=\"Image\" /></par>\r\n");
            }

            if (partText != null)
            {
                smilBuilder.Append("<par><img src=\"" + partText.Parameters["Content-Location"] +
                                   "\" region=\"Text\" /></par>\r\n");
            }

            smilBuilder.Append("</body>\r\n");
            smilBuilder.Append("</smil>\r\n");


            // Add prepared parts into MMS message
            partSmil.Data = Encoding.UTF8.GetBytes(smilBuilder.ToString());
            mes.Parts.Add(partSmil);

            if (partImage != null)
            {
                mes.Parts.Add(partImage);
            }

            if (partText != null)
            {
                mes.Parts.Add(partText);
            }


            MM7Client client = _clientCreator.Create();

            SubmitResponse resp = await client.Submit(mes, token).ConfigureAwait(false);

            return resp;

        }


        public async Task<GenericResponse> SendMMSFromSMIL(string recipient, string subject, string smilFilePath, CancellationToken token = default)
        {

            if (!File.Exists(smilFilePath))
            {
                throw new ArgumentException($"SMIL file {smilFilePath} not found. Please specify correct file path.");
            }

            MM7Client client = _clientCreator.Create();

            SubmitRequest req = client.CreateSubmitRequest();

            req.Recipients.To.Add(recipient);

            req.MessageClass = MessageClass.Personal;
            req.DeliveryReport = true;
            req.Subject = subject;
            req.Priority = Priorities.Normal;
            req.ChargedParty = ChargedParty.Sender;
            req.ContentHref = "/mms/mm7/mm7client";
            req.TransactionID = DateTime.Now.Ticks.ToString();


            // Submit MMS Message as VASP
            SubmitResponse resp = await client.Submit(smilFilePath, req, token).ConfigureAwait(false);
            // Check response status
            return resp;

        }



        /// <summary>
        /// Get ContentType string for specified ImageFormat
        /// </summary>
        /// <param name="format"></param>
        /// <returns></returns>
        private string GetContentTypeByImageFormat(ImageFormat format)
        {
            string ctype = null;

            if (format.Equals(ImageFormat.Jpeg))
            {
                ctype = "image/jpeg";
            }
            else if (format.Equals(ImageFormat.Gif))
            {
                ctype = "image/gif";
            }
            else if (format.Equals(ImageFormat.Png))
            {
                ctype = "image/png";
            }
            else if (format.Equals(ImageFormat.Bmp))
            {
                ctype = "image/bmp";
            }

            return ctype;
        }
    }
}
