﻿using System;
using System.Collections.Generic;
using System.Collections.Specialized;
using System.IO;
using System.Security.Claims;
using System.Security.Principal;
using System.Threading.Tasks;
using Inetlab.MMS;
using Microsoft.AspNetCore.Http;
using Microsoft.Extensions.Primitives;

namespace MMSServer.ASPNETCore
{
    public class HttpContextWrapper : IHttpContext
    {
        private readonly HttpContext _httpContext;
        private readonly HttpResponseWrapper _httpResponse;
        private readonly HttpRequestWrapper _httpRequest;

        public HttpContextWrapper(HttpContext httpContext)
        {
            _httpContext = httpContext;
            _httpResponse = new HttpResponseWrapper(_httpContext.Response);
            _httpRequest = new HttpRequestWrapper(_httpContext.Request);
        }

        public IHttpResponse Response
        {
            get
            {
                return _httpResponse;
            }
        }

        public IPrincipal User
        {
            get { return _httpContext.User; }
            set
            {
                if (value == null) _httpContext.User = null;

                var claimsPrincipal = value as ClaimsPrincipal;
                if (claimsPrincipal == null) throw new NotSupportedException($"Principal type {value.GetType().FullName} is not supported. Use {typeof(ClaimsPrincipal).FullName}.");

                _httpContext.User = claimsPrincipal;
            }
        }

        public IHttpRequest Request
        {
            get
            {
                return _httpRequest;
            }
        }
    }


    public class HttpRequestWrapper : IHttpRequest
    {
        private readonly HttpRequest _httpRequest;
        private NameValueCollection _headers;

        public HttpRequestWrapper(HttpRequest httpRequest)
        {
            _httpRequest = httpRequest;
            _headers = new NameValueCollection();
            foreach (KeyValuePair<string, StringValues> header in _httpRequest.Headers)
            {
                _headers.Add(header.Key, header.Value.ToString());
            }
        }

        public Uri Url
        {
            get
            {
                var uriBuilder = new UriBuilder
                {
                    Scheme = _httpRequest.Scheme,
                    Host = _httpRequest.Host.Host,
                    Port = _httpRequest.Host.Port.GetValueOrDefault(80),
                    Path = _httpRequest.Path.ToString(),
                    Query = _httpRequest.QueryString.ToString()
                };
                return uriBuilder.Uri;
            }
        }

        public string Method
        {
            get { return _httpRequest.Method; }
        }

        public string UserHostAddress
        {
            get
            {
                return _httpRequest.HttpContext.Connection.RemoteIpAddress.ToString();
            }
        }

        public long ContentLength
        {
            get { return _httpRequest.ContentLength.GetValueOrDefault(); }
        }

        public Stream InputStream
        {
            get { return _httpRequest.Body; }
        }

        public NameValueCollection Headers
        {
            get { return _headers; }
        }

        public string ContentType
        {
            get { return _httpRequest.ContentType; }
        }
    }

    public class HttpResponseWrapper : IHttpResponse
    {
        private readonly HttpResponse _httpResponse;

        public HttpResponseWrapper(HttpResponse httpResponse)
        {
            _httpResponse = httpResponse;
        }



        public int StatusCode
        {
            get { return _httpResponse.StatusCode; }
            set { _httpResponse.StatusCode = value; }
        }

        public string ContentType
        {
            get { return _httpResponse.ContentType; }
            set { _httpResponse.ContentType = value; }
        }

        public long ContentLength
        {
            get { return _httpResponse.ContentLength.GetValueOrDefault(); }
            set { _httpResponse.ContentLength = value; }
        }

        public Stream OutputStream
        {
            get { return _httpResponse.Body; }
        }

        public void Close()
        {

        }

        public void AddHeader(string name, string value)
        {
            _httpResponse.Headers.Add(name, value);
        }

        public Task Write(string text)
        {
            return _httpResponse.WriteAsync(text);
        }
    }
}